@@ -13,6 +13,8 @@
 ##############################################################################
 """Grok components"""
 
+from operator import itemgetter
+
 from zope import component, interface
 from zope.viewlet.manager import ViewletManagerBase
 from zope.viewlet.viewlet import ViewletBase
@@ -31,35 +33,22 @@ class ViewletManager(ViewletManagerBase):
         self.request = request
         self.view = view
         self.__name__ = self.__view_name__
-        self.static = component.queryAdapter(
-            self.request,
-            interface.Interface,
-            name=self.module_info.package_dotted_name,
-            )
+        static_name = getattr(self, '__static_name__', None)
+        if static_name is not None:
+            self.static = component.queryAdapter(
+                self.request,
+                interface.Interface,
+                name=static_name)
+        else:
+            self.static = None
 
     def sort(self, viewlets):
         """Sort the viewlets.
 
         ``viewlets`` is a list of tuples of the form (name, viewlet).
         """
-        # In Grok, the default order of the viewlets is determined by
-        # util.sort_components. util.sort_components() however expects
-        # a list of just components, but sort() is supposed to deal
-        # with a list of (name, viewlet) tuples.
-        # To handle this situation we first store the name part on the
-        # viewlet, then use util.sort_components() and then "unpack"
-        # the name from the viewlet and recreate the list of (name,
-        # viewlet) tuples, now in the correct order.
-        s_viewlets = []
-        for name, viewlet in viewlets:
-            # Stuff away viewlet name so we can later retrieve it.
-            # XXX We loose name information in case the same viewlet
-            # is in the viewlets list twice, but with a different
-            # name. Most probably this situation doesn't occur.
-            viewlet.__viewlet_name__ = name
-            s_viewlets.append(viewlet)
-        s_viewlets = util.sort_components(s_viewlets)
-        return [(viewlet.__viewlet_name__, viewlet) for viewlet in s_viewlets]
+        # Sort viewlets following grok.order rule.
+        return util.sort_components(viewlets, key=itemgetter(1))
 
     def default_namespace(self):
         namespace = {}
@@ -99,11 +88,14 @@ class Viewlet(ViewletBase):
         self.view = view
         self.viewletmanager = manager
         self.__name__ = self.__view_name__
-        self.static = component.queryAdapter(
-            self.request,
-            interface.Interface,
-            name=self.module_info.package_dotted_name,
-            )
+        static_name = getattr(self, '__static_name__', None)
+        if static_name is not None:
+            self.static = component.queryAdapter(
+                self.request,
+                interface.Interface,
+                name=static_name)
+        else:
+            self.static = None
 
     def default_namespace(self):
         namespace = {}
